/**
 * Recursive Tree
 * by Daniel Shiffman.  
 * 
 * Renders a simple tree-like structure via recursion. 
 * The branching angle is calculated as a function of 
 * the horizontal mouse location. Move the mouse left
 * and right to change the angle.
 */
 
// importing the processing serial class
import processing.serial.*;

// variables for serial connection, portname and baudrate have to be set 
  Serial port;
  String portname = "COM2";  
  int baudrate = 9600;
  int value = 0; 
  String buf=""; 
  int value1=0;   
  int xpos;
  boolean output;
  int wave;
 
float theta;   

void serialEvent(int serial){
  // if serial event is not a line break 
  if(serial!=10) {        
    // add event to buffer
    buf += char(serial);          
    } else {
    // if serial is line break set value1 to buff and clear it
    value1 = int(buf);
    buf="";
    }
    // convert value1 to xpos
    xpos = constrain ( ( ((value1-100)*(width+100))/width )  ,0, width-12); 
    if(output) println("xpos: "+xpos);
}

void setup() {
  size(640, 360);
  smooth();
  
  // establish serial port connection      
  port = new Serial(this, portname, baudrate);
  println(port); 
}

void draw() {
// listen to serial port and trigger serial event  
  while(port.available() > 0){
        value = port.read();
        serialEvent(value);
    }
  
  wave++;
  
  background(0);
  frameRate(30);
  stroke(255);
  // Let's pick an angle 0 to 90 degrees based on the mouse position
  //float a = (mouseX / (float) width) * 90f;
  float a = (xpos / (float) width) * 90f;
  // Convert it to radians
  theta = radians(a);
  // Start the tree from the bottom of the screen
  translate(width/2,height);
  // Draw a line 120 pixels
  line(0,0,0,-120);
  // Move to the end of that line
  translate(0,-120);
  // Start the recursive branching!
  branch(120);

}

void branch(float h) {
  // Each branch will be 2/3rds the size of the previous one
  h *= 0.66;
  
  // All recursive functions must have an exit condition!!!!
  // Here, ours is when the length of the branch is 2 pixels or less
  if (h > 2) {
    pushMatrix();    // Save the current state of transformation (i.e. where are we now)
    rotate(theta+sin(wave/10.0)*0.05);   // Rotate by theta
    line(0, 0, 0, -h);  // Draw the branch
    translate(0, -h); // Move to the end of the branch
    branch(h);       // Ok, now call myself to draw two new branches!!
    popMatrix();     // Whenever we get back here, we "pop" in order to restore the previous matrix state
    
    // Repeat the same thing, only branch off to the "left" this time!
    pushMatrix();
    rotate(-theta);
    line(0, 0, 0, -h);
    translate(0, -h);
    branch(h);
    popMatrix();
  }
}


